﻿// *****************************************************************
// *                                                               *
// *       Copyright (c) Fast Corporation, 2010                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

// -----------------------------------------------------------------
// FVIL Sample Program
// メインダイアログ
//
// 作成者 : N.Moriguchi
//
// 履歴 : 2012/05/16 初版
//
// FVC07 1ch only Sample Program
// -----------------------------------------------------------------
namespace FVC07_1chSample
{
    public partial class Form_FVC071CHSAMPLE : Form
    {

        //画像メモリ格納領域
        private FVIL.Data.CFviImage m_image = new FVIL.Data.CFviImage();
        //FVC07クラス
        private FVIL.Video.CFviVideoFVC07 m_video = null;
        //処理時間計測クラス
        private FVIL.CFviTimeCounter m_timer = new FVIL.CFviTimeCounter();

        //iniファイル名（フルパス）
        public string m_strIniFilePath;
        //ビデオデバイスのオープン状態
        private bool m_flgVideoOpened;
        //フリーランの状態
        private bool m_flgFreerun;
        
        public Form_FVC071CHSAMPLE()
        {
            InitializeComponent();
        }

        /// <summary>
        /// フォーム表示開始
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 使用許諾を確認します。
        /// </remarks>
        private void Form1_Shown(object sender, EventArgs e)
        {
            dialogLicence licence_dialog = new dialogLicence();
            if ((licence_dialog.ShowDialog(this) != DialogResult.OK))
            {
                this.Close();
                //使用許諾が得られない場合はサンプルプログラムの動作を終了
                return;
            }

             //iniファイル選択
            fnSetIniFile();

            // 描画imageのセット
            cFviImageView1.Image = m_image;

        }

        /// <summary>
        /// メインフォーム初期化
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// メインフォームの初期化処理。
        /// </remarks>
        private void Form1_Load(object sender, EventArgs e)
        {
            // FVILの初期化
            // アプリケーションを起動した後、 この関数を呼び出してライブラリを初期化してください。 
            //  この時、ライセンスの認証が行われます。試用期間中であっても、この処理は必ず行う必要が有ります。 
            // 初期化を行わなかった場合は、プロテクトされたクラスの実行ができません。
            FVIL._SetUp.InitVisionLibrary();

            //状態変数の初期化
            m_flgVideoOpened = false;
            m_flgFreerun = false;

            //計測時間テキストボックス初期化
            TextBoxTime.Text = "0";

            //エンコーダパラメータコンボボックス初期化
            ComboBoxEncMode.Items.Clear();
            ComboBoxEncMode.Items.Add("エンコーダを使用しない");
            ComboBoxEncMode.Items.Add("エンコーダスキャンモード");
            ComboBoxEncMode.Items.Add("エンコーダライン選択モード");
            ComboBoxEncMode.SelectedIndex = 0;


            ComboBoxSignal.Items.Clear();
            ComboBoxSignal.Items.Add("AB相");
            ComboBoxSignal.Items.Add("A相");
            ComboBoxSignal.SelectedIndex = 0;

            ComboBoxMlt.Items.Clear();
            ComboBoxMlt.Items.Add("1倍");
            ComboBoxMlt.Items.Add("2倍");
            ComboBoxMlt.Items.Add("4倍");
            ComboBoxMlt.SelectedIndex = 0;

            ComboBoxStart.Items.Clear();
            ComboBoxStart.Items.Add("CPU");
            ComboBoxStart.Items.Add("外部トリガ");
            ComboBoxStart.Items.Add("Z相");
            ComboBoxStart.SelectedIndex = 0;
            
            ComboBoxPulse.Items.Clear();
            ComboBoxPulse.Items.Add("CW");
            ComboBoxPulse.Items.Add("CCW");
            ComboBoxPulse.SelectedIndex = 0;

            ComboBoxInit.Items.Clear();
            ComboBoxInit.Items.Add("初期化する");
            ComboBoxInit.Items.Add("初期化しない");
            ComboBoxInit.SelectedIndex = 0;

            ComboBoxCntEnb.Items.Clear();
            ComboBoxCntEnb.Items.Add("無効");
            ComboBoxCntEnb.Items.Add("相対位置エンコーダ");
            ComboBoxCntEnb.Items.Add("絶対位置エンコーダ");
            ComboBoxCntEnb.SelectedIndex = 0;

            ComboBoxSwapAB.Items.Clear();
            ComboBoxSwapAB.Items.Add("無効");
            ComboBoxSwapAB.Items.Add("A/B相入替");
            ComboBoxSwapAB.SelectedIndex = 0;
            
            //トリガモードコンボボックス初期化
            ComboBoxTrigger.Items.Clear();
            ComboBoxTrigger.Items.Add("ノーマル");
            ComboBoxTrigger.Items.Add("内部トリガ");
            ComboBoxTrigger.Items.Add("外部トリガ");
            ComboBoxTrigger.Items.Add("外部トリガ2");
            ComboBoxTrigger.SelectedIndex = 0;

            //サイクルタイムモードコンボンボボックス初期化
            ComboBoxCycleTimeMode.Items.Clear();
            ComboBoxCycleTimeMode.Items.Add("トリガ幅を含まない時間");
            ComboBoxCycleTimeMode.Items.Add("トリガ幅を含む時間");
            ComboBoxCycleTimeMode.SelectedIndex = 0;

        }

        /// <summary>
        /// iniファイル選択関数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// iniファイルの選択をします。
        /// </remarks>
        private void fnSetIniFile()
        {
            //iniファイル設定画面
            Form_BROWSEIN formbroseini = new Form_BROWSEIN();

            //iniファイルの選択
            m_strIniFilePath = "";
            if ((formbroseini.ShowDialog(this) == DialogResult.OK))
            {
                m_strIniFilePath = formbroseini.strIniFilePath;
            }

            if (m_strIniFilePath=="")
            {
                this.Close();
                return;
            }

            if ((m_strIniFilePath.Length > 0))
            {
                try
                {
                    //ビデオデバイスがオープンされていた場合はクローズする
                    if (m_video != null)
                    {
                        if (m_video.IsOpen())
                        {
                            m_video.Close();
                            m_video = null;
                            m_flgVideoOpened = false;
                        }
                    }

                    //
                    //指定されたカメラ設定ファイルに該当するビデオ入力クラスのインスタンスを生成し、 Open 関数と LoadIniFile(String) 関数を実行して返します。 
                    //iniファイルを指定して、デバイスオープン
                    m_video = (FVIL.Video.CFviVideoFVC07)FVIL.Video.Function.CreateInstance(m_strIniFilePath);
                    m_flgVideoOpened = true;

                    //VideoConfFileの変更後、OpenDeviceを行うと、トリガーモードの変更が伴います
                    //弊社キャプチャボードはトリガーモードの設定後にセットアップタイムが必要になります｡
                    //本プログラムのセットアップタイムは必要最大値である350msを設定しています。
                    //キャプチャボードによってこの時間は異なります。
                    System.Threading.Thread.Sleep(350);

                    //取込画像を正規化
                    m_video.ChangeImageSize(m_image);

                    //ステータス表示の更新
                    //エンコーダパラメータ取得
                    ComboBoxSignal.SelectedIndex = (int)m_video.EncoderParam.Signal;
                    ComboBoxMlt.SelectedIndex = (int)m_video.EncoderParam.Mlt;
                    ComboBoxStart.SelectedIndex = (int)m_video.EncoderParam.Start;
                    ComboBoxPulse.SelectedIndex = (int)m_video.EncoderParam.Pulse;
                    NumericUpDownComp1.Value = m_video.EncoderParam.Comp1;
                    NumericUpDownComp2.Value = m_video.EncoderParam.Comp2;
                    ComboBoxInit.SelectedIndex = (int)m_video.EncoderParam.Init;
                    ComboBoxCntEnb.SelectedIndex = (int)m_video.EncoderParam.Cntenb;
                    ComboBoxSwapAB.SelectedIndex = (int)m_video.EncoderParam.Swapab;
                    
                    //トリガモード取得
                    ComboBoxTrigger.SelectedIndex = (int)m_video.Trigger;
                    ComboBoxCycleTimeMode.SelectedIndex = (int)m_video.TriggerCycleTime.Mode;
                    NumericUpDownExposure.Value = m_video.ExposureTime;
                    NumericUpDownCycleTime.Value = (int)m_video.TriggerCycleTime.Time;

                    //ラインセンサモード取得
                    if (m_video.LineSensorMode.LsEnable == 1)
                    {
                        CheckLineSensorMode.Checked = true;
                    }
                    else
                    {
                        CheckLineSensorMode.Checked = false;
                    }
                    
                    //エンコーダモード取得
                    ComboBoxEncMode.SelectedIndex = (int)m_video.LineSensorMode.EncEnable;
                    

                    //エンコーダモード時、エンコーダパラメータ変更可
                    if (m_video.LineSensorMode.EncEnable != 0)
                        GroupEncorderParam.Enabled = true;
                    else
                        GroupEncorderParam.Enabled = false;

                }
                catch (FVIL.CFviException ex)
                {
                    //エラー処理
                    MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (System.Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
            {
                //iniファイルが未選択の場合はサンプルプログラムの動作を終了
                this.Close();
                return;
            }

        }

        /// <summary>
        /// Grab Oneボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ取り込み。
        /// </remarks>
        private void ButtonGrabOne_Click(object sender, EventArgs e)
        {
            double msec = 0;

            try
            {
                m_timer.Start();
                //時間測定開始
                //カメラ取り込み
                m_video.GrabImageSync(m_image);
                //時間測定終了
                msec = m_timer.Stop();
                //測定時間表示
                TextBoxTime.Text = msec.ToString("0.000");
                //再描画
                cFviImageView1.Refresh();
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Grab Continueボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// フリーラン開始・停止。
        /// </remarks>
        private void ButtonGrabContinue_Click(object sender, EventArgs e)
        {
            if (m_flgFreerun == true)
            {
                ButtonGrabContinue.Text = "Grab Continue";
                ButtonGrabOne.Enabled = true;
                m_flgFreerun = false;
                timer1.Enabled = false;
            }
            else
            {
                ButtonGrabContinue.Text = "Stop";
                ButtonGrabOne.Enabled = false;
                m_flgFreerun = true;
                timer1.Interval = 100;
                timer1.Start();
            }
            Application.DoEvents();

        }

        /// <summary>
        /// フリーラン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 連続カメラ取り込み。
        /// </remarks>
        private void timer1_Tick(object sender, EventArgs e)
        {
            double msec = 0;

            timer1.Stop();

            try
            {
                if (m_flgVideoOpened == true)
                {
                    //時間測定開始
                    m_timer.Start();
                    //カメラ取り込み
                    m_video.GrabImageSync(m_image);
                    //時間測定終了
                    msec = m_timer.Stop();
                    //測定時間表示
                    TextBoxTime.Text = msec.ToString("0.000");
                }
                timer1.Enabled = true;

            }
            catch (FVIL.CFviException ex)
            {
                ButtonGrabContinue.Text = "Grab Continue";
                ButtonGrabOne.Enabled = true;
                m_flgFreerun = false;
                timer1.Enabled = false;
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                ButtonGrabContinue.Text = "Grab Continue";
                ButtonGrabOne.Enabled = true;
                m_flgFreerun = false;
                timer1.Enabled = false;
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// Trigger Modeコンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ノーマルモード・内部トリガモード・外部トリガモードの選択。
        /// </remarks>
        private void ComboBoxTrigger_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    m_video.Trigger = (uint)ComboBoxTrigger.SelectedIndex;
                    //弊社キャプチャボードはトリガーモードの設定後にセットアップタイムが必要になります｡
                    //本プログラムのセットアップタイムは必要最大値である350msを設定しています。
                    //キャプチャボードによってこの時間は異なります。
                    System.Threading.Thread.Sleep(350);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// CycleTime Mode変更
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// サイクルタイムのモード変更。
        /// </remarks>
        private void ComboBoxCycleTimeMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    FVIL.Video.CFviTriggerCycleTime trgcycle = m_video.TriggerCycleTime;
                    trgcycle.Mode = (uint)ComboBoxCycleTimeMode.SelectedIndex;
                    m_video.TriggerCycleTime = trgcycle;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Video Exposure値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ露光時間の設定。
        /// </remarks>
        private void NumericUpDownExposure_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    m_video.ExposureTime = (uint)NumericUpDownExposure.Value;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Cycle Time値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// サイクルタイムの変更。
        /// </remarks>
        private void NumericUpDownCycleTime_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    FVIL.Video.CFviTriggerCycleTime trgcycle = m_video.TriggerCycleTime;
                    trgcycle.Time = (int)NumericUpDownCycleTime.Value;
                    m_video.TriggerCycleTime = trgcycle;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// ラインセンサモードチェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ラインセンサモードの設定（エリア　Or ラインセンサを指定する）。
        /// </remarks>
        private void CheckLineSensorMode_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    FVIL.Video.CFviLineSensorMode linemode = m_video.LineSensorMode;
                    linemode.LsEnable = System.Convert.ToUInt32(CheckLineSensorMode.Checked);
                    m_video.LineSensorMode = linemode;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// エンコーダパラメータ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// AB相の設定( 0:AB相、1:A相 )。
        /// </remarks>
        private void ComboBoxSignal_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    FVIL.Video.CFviEncoderParam2 encmode = m_video.EncoderParam;
                    encmode.Signal = (uint)ComboBoxSignal.SelectedIndex;
                    m_video.EncoderParam = encmode;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// エンコーダパラメータ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// サンプリング速度の設定 ( 0:1倍、1:2倍、2:4倍 )。
        /// </remarks>
        private void ComboBoxMlt_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    FVIL.Video.CFviEncoderParam2 encmode = m_video.EncoderParam;
                    encmode.Mlt = (uint)ComboBoxMlt.SelectedIndex;
                    m_video.EncoderParam = encmode;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// エンコーダパラメータ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// エンコーダ起動モードの設定( 0:CPU、1:外部トリガ、2:Z相 )。
        /// </remarks>
        private void ComboBoxStart_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    FVIL.Video.CFviEncoderParam2 encmode = m_video.EncoderParam;
                    encmode.Start = (uint)ComboBoxStart.SelectedIndex;
                    m_video.EncoderParam = encmode;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// エンコーダパラメータ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// パルスの方向の設定 ( 0:CW、1:CCW )。
        /// </remarks>
        private void ComboBoxPulse_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    FVIL.Video.CFviEncoderParam2 encmode = m_video.EncoderParam;
                    encmode.Pulse = (uint)ComboBoxPulse.SelectedIndex;
                    m_video.EncoderParam = encmode;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// エンコーダパラメータ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// エンコーダカウンタ初期化の設定( 0:初期化する、1:初期化しない )。
        /// </remarks>
        private void ComboBoxInit_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    FVIL.Video.CFviEncoderParam2 encmode = m_video.EncoderParam;
                    encmode.Init = (uint)ComboBoxInit.SelectedIndex;
                    m_video.EncoderParam = encmode;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// エンコーダパラメータ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 比較レジスタ1の設定。
        /// </remarks>
        private void NumericUpDownComp1_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    FVIL.Video.CFviEncoderParam2 encmode = m_video.EncoderParam;
                    encmode.Comp1 = (int)NumericUpDownComp1.Value;
                    m_video.EncoderParam = encmode;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// エンコーダパラメータ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 比較レジスタ2の設定。
        /// </remarks>
        private void NumericUpDownComp2_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    FVIL.Video.CFviEncoderParam2 encmode = m_video.EncoderParam;
                    encmode.Comp2 = (uint)NumericUpDownComp2.Value;
                    m_video.EncoderParam = encmode;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }
        /// <summary>
        /// エンコーダタイプ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// エンコーダタイプの設定。
        /// </remarks>
        private void ComboBoxCntEnb_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    FVIL.Video.CFviEncoderParam2 encmode = m_video.EncoderParam;
                    encmode.Cntenb = (uint)ComboBoxCntEnb.SelectedIndex;
                    m_video.EncoderParam = encmode;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }
        /// <summary>
        /// AB相入替
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// AB相入替の設定。
        /// </remarks>
        private void ComboBoxSwapAB_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    FVIL.Video.CFviEncoderParam2 encmode = m_video.EncoderParam;
                    encmode.Swapab = (uint)ComboBoxSwapAB.SelectedIndex;
                    m_video.EncoderParam = encmode;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Sendボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// シリアル通信でカメラ設定を変更します。
        /// </remarks>
        private void ButtonSendSIO_Click(object sender, EventArgs e)
        {
            try
            {
                List<byte> senddata, revdata;
                Byte[] byterevdata;
                int i;

                //送信データを取得
                senddata = new List<byte>(System.Text.Encoding.ASCII.GetBytes(TextBoxSendData.Text));

                //シリアルポートをオープン
                m_video.Sio.Open();

                //データ送信
                revdata = m_video.Sio.SendData(senddata, 0);

                //受信データをByteに変換
                byterevdata = revdata.ToArray();

                //受信データを表示
                TextBoxRecieveData.Text = "";
                for (i = 0; i < revdata.Count; i++)
                {
                    TextBoxRecieveData.Text = System.Text.Encoding.ASCII.GetString(byterevdata);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                m_video.Sio.Close();
            }

        }

        /// <summary>
        /// x1ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 等倍で表示します。
        /// </remarks>
        private void ButtonMag1_Click(object sender, EventArgs e)
        {
            //画像の表示倍率の取得または設定を行います。1.0と指定すると等倍で表示します。
            //倍率値は、0.5とした場合は、幅・高さ共に1/2となり、面積が1/4になります。
            //3.0とした場合は、幅・高さ共に3倍となり、面積が9倍になります。
            cFviImageView1.Display.Magnification = 1.0; //表示倍率の設定
            cFviImageView1.Refresh();                   //再描画

        }

        /// <summary>
        /// Fitボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します。
        /// </remarks>
        private void ButtonFit_Click(object sender, EventArgs e)
        {
            //画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します
            cFviImageView1.FitImageSize();                   //表示倍率を調整
            cFviImageView1.Refresh();                        //再描画

        }

        /// <summary>
        /// ＋ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率に0.5加えた倍率で表示します。
        /// </remarks>
        private void ButtonPlus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = cFviImageView1.Display.Magnification;
            //表示倍率の設定
            Value = Value + 0.5;
            cFviImageView1.Display.Magnification = Value;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// －ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率から0.5引いた倍率で表示します。ただし、倍率が０以下になる場合は倍率0.1で表示します。
        /// </remarks>
        private void ButtonMinus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = cFviImageView1.Display.Magnification;
            Value = Value - 0.5;
            if (Value <= 0)
            {
                //０以下を設定するとエラーになるので0.1を設定する
                Value = 0.1;
            }
            //表示倍率の設定
            cFviImageView1.Display.Magnification = Value;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// Ascopeチェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ASCOPEの表示を切替えます。
        /// </remarks>
        private void CheckBoxAscope_CheckedChanged(object sender, EventArgs e)
        {
            //ASCOPEオーバレイの設定
            if (CheckBoxAscope.Checked)
            {
                //ASCOPEの表示を有効にする。
                cFviImageView1.Display.OverlayAscope.Active = true;
                cFviImageView1.Display.OverlayAscope.Enable = true;
            }
            else
            {
                //ASCOPEの表示を無効にする。
                cFviImageView1.Display.OverlayAscope.Enable = false;
                cFviImageView1.Display.OverlayAscope.Active = false;
            }
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// ReLoad Ini Fileボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ビデオ設定用iniファイルの再設定。
        /// </remarks>
        private void ButtonReLoadIni_Click(object sender, EventArgs e)
        {
            //フリーラン中ならば、停止する。
	        if (m_flgFreerun == true) {
		        ButtonGrabContinue.Text = "Grab Continue";
		        ButtonGrabOne.Enabled = true;
		        m_flgFreerun = false;
		        timer1.Enabled = false;
		        Application.DoEvents();
	        }

        	//iniファイル選択
	        fnSetIniFile();

        }

        /// <summary>
        /// Save Imageボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 表示画像保存。
        /// </remarks>
        private void ButtonSaveImage_Click(object sender, EventArgs e)
        {
            
            SaveFileDialog ofdlg = new SaveFileDialog();

            ofdlg.Filter = "画像ファイル(*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat)|*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat";
            ofdlg.Filter += "|Bitmap ファイル (*.bmp;*.dib)|*.bmp;*.dib";
            ofdlg.Filter += "|Png ファイル (*.png)|*.png";
            ofdlg.Filter += "|Jpeg ファイル (*.jpg;*.jpeg)|*.jpg;*.jpeg";
            ofdlg.Filter += "|Tiff ファイル (*.tif;*.tiff)|*.tif;*.tiff";
            ofdlg.Filter += "|Raw ファイル (*.raw)|*.raw";
            ofdlg.Filter += "|すべてのファイル (*.*)|*.*";
            ofdlg.OverwritePrompt = true;

            if (ofdlg.ShowDialog(this) == DialogResult.OK)
            {
                try
                {
                    //この関数は、簡易的に画像ファイルの保存を行う為のものです。 
                    //この関数で指定できないパラメータを指定する必要があれば、 各画像ファイルアクセスクラスを直接使用してください。 
                    FVIL.File.Function.SaveImageFile(ofdlg.FileName, cFviImageView1.Image);

                }
                catch (FVIL.CFviException ex)
                {
                    System.String msg = System.String.Format("ライブラリが例外を発行しました。code={0} func={1}", ex.ErrorCode, ex.Function);
                    switch ((ex.ErrorCode))
                    {
                        case FVIL.ErrorCode.NOT_SUPPORTED:
                        case FVIL.ErrorCode.INVALID_OBJECT:
                            msg += "\r\n" + "サポートされていない画像オブジェクトです。";
                            msg += "\r\n" + "Raw 形式で保存してください。";
                            break;
                    }
                    //エラー処理
                    MessageBox.Show(msg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// 終了処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// アプリケーション終了処理。
        /// </remarks>
        private void Form_FVC071CHSAMPLE_FormClosing(object sender, FormClosingEventArgs e)
        {
            // ビデオデバイスがオープンされていた場合はクローズする
            if (m_flgVideoOpened) 
            {
                m_video.Close();
                m_video = null;
                m_flgVideoOpened = false;
            }
        }

        /// <summary>
        /// エンコーダモードの設定
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// エンコーダの使用有無を設定
        /// </remarks>
        private void ComboBoxEncMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    FVIL.Video.CFviLineSensorMode linemode = m_video.LineSensorMode;
                    linemode.EncEnable = (uint)ComboBoxEncMode.SelectedIndex;
                    m_video.LineSensorMode = linemode;

                    //エンコーダモード時、エンコーダパラメータ変更可
                    if (m_video.LineSensorMode.EncEnable != 0)
                        GroupEncorderParam.Enabled = true;
                    else
                        GroupEncorderParam.Enabled = false;
                }



            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }


        }

    }
}